package com.shixing.sxedit;

import com.shixing.sxedit.types.SXEffectTimeExtendType;
import com.shixing.sxedit.types.SXEffectTimeFollowType;
import com.shixing.sxedit.types.SXEffectType;

/**
 * 特效基类
 * */

public abstract class SXEffect {



    /**
     * 获取特效 Id
     *
     * @return 特效 id
     * */
    public String getEffectId() {
        return mEffectId;
    }

    /**
     * 获取特效类型
     *
     * @return {@link SXEffectType}
     * */
    public abstract SXEffectType getType();

    /**
     *  获取特效开始时间
     *
     *  <p>
     * 不同时间跟随方式start time的参照时间是不一样的，跟随方式祥见 {@link SXEffectTimeFollowType} 类型说明
     *  </p>
     *
     * @return 秒
     */
    public double getStartTime() {
        if (getNativeEffect() > 0) {
            return nStartTime(mNativeEffect);
        }
        return 0;
    }

    /**
     * 设置开始时间
     *
     *  <p>
     * 不同时间跟随方式start time的参照时间是不一样的，跟随方式祥见 {@link SXEffectTimeFollowType} 类型说明
     *  </p>
     *
     * @param startTime
     */
    public void setStartTime(double startTime) {
        if (getNativeEffect() > 0) {
            nSetStartTime(mNativeEffect, startTime);
        }
    }

    /**
     * 效果的持续时长，时间跟随方式为 FollowWholeProcess 时无效
     *
     * @return 时长，单位为秒
     */
    public double getDuration() {
        if (getNativeEffect() > 0) {
            return nDuration(mNativeEffect);
        }
        return 0;
    }

    /**
     * 设置时长
     * <p>
     * 时间跟随方式为 {@link SXEffectTimeFollowType#FollowWholeProcess} 时无效
     * </p>
     *
     * @param duration 秒
     */
    public void setDuration(double duration) {
        if (getNativeEffect() > 0) {
            nSetDuration(mNativeEffect, duration);
        }
    }

    /**
     * 设置特效倍速，默认为1倍速
     *
     * <p>
     * 1. 仅扩展方式为 {@link SXEffectTimeExtendType#StretchAndLoop} 时有效
     * </p>
     *
     *
     * @param speed 倍速
     */
    public void setSpeed(float speed) {
        if (getNativeEffect() > 0) {
            nSetSpeed(mNativeEffect, speed);
        }
    }

    /**
     * 获取特效倍速
     *
     * @return 特效倍速
     * */
    public float getSpeed() {
        if (getNativeEffect() > 0) {
            return nSpeed(mNativeEffect);
        }
        return 0;
    }

    /**
     * 获取一次循环的时长，时长由倍速转化，因此未设置动画资源时返回值为0
     *
     * <p>
     *
     * 1. 仅扩展方式为kStretchAndLoop时有效
     * </p>
     *
     * @return 一次循环的时长
     */
    public double getDurationOfOneCycle() {
        if (getNativeEffect() > 0) {
            return nDurationOfOneCycle(mNativeEffect);
        }
        return 0;
    }

    /**
     * 设置一次循环的时长，时长会转化为倍速，因此未设置动画资源时此设置无效
     * <p>
     * 1. 仅扩展方式为kStretchAndLoop时有效
     * </p>
     *
     * @param duration 一次循环的时长
     */
    public void setDurationOfOneCycle(double duration) {
        if (getNativeEffect() > 0) {
            nSetDurationOfOneCycle(mNativeEffect, duration);
        }
    }

    /**
     * 设置此效果是跟随轨道时间方式，默认为跟随轨道开始显示时间即 {@link SXEffectTimeFollowType#FollowDisplayTime}
     *
     * @param type 跟随轨道时间方式
     */
    public void setEffectFollowType(SXEffectTimeFollowType type) {
        if (getNativeEffect() > 0) {
            nSetFollowType(mNativeEffect, type.ordinal());
        }
    }

    /**
     * 获取效果跟随轨道时间方式
     * <p>
     * 1. 例如跟随轨道开始时间与轨道一同开始、跟随轨道的结束时间与轨道一同结束等，
     * 默认跟随方式为 {@link SXEffectTimeFollowType#FollowDisplayTime}
     * </p>
     *
     * @return 跟随轨道时间方式
     */
    public SXEffectTimeFollowType getEffectFollowType() {
        if (getNativeEffect() > 0) {
            int type = nFollowType(mNativeEffect);
            return SXEffectTimeFollowType.values()[type];
        }
        return null;
    }

    /**
     * 设置效果在显示时间区间内的扩展方式
     * <p>
     * 1. 扩展方式决定当特效时间超出特效自身的时长时如何扩展
     * 2. 默认扩展方式为 {@link SXEffectTimeExtendType#Stretch}
     * </p>
     * @param type 扩展类型
     */
    public void setEffectExtendType(SXEffectTimeExtendType type) {
        if (getNativeEffect() > 0) {
            nSetExtendType(mNativeEffect, type.ordinal());
        }
    }

    /**
     * 获取特效的时间扩展方式
     *
     * @description 扩展方式决定当特效时间超出特效自身的时长时如何扩展，默认扩展方式为 {@link SXEffectTimeExtendType#Stretch}
     *
     * @return 扩展方式
     */
    public SXEffectTimeExtendType getExtendType() {
        if (getNativeEffect() > 0) {
            return SXEffectTimeExtendType.values()[nExtendType(mNativeEffect)];
        }
        return null;
    }


    @Override
    public boolean equals(Object obj) {
        return obj instanceof SXEffect && mEffectId.equals(((SXEffect) obj).mEffectId);
    }


    protected String mEffectId;
    protected long mNativeManager;
    protected String mTrackId;
    protected long mNativeEffect;

    public SXEffect(long nativeEffect, long nativeManager, String trackID) {
        mEffectId = nGetEffectId(nativeEffect);
        mNativeManager = nativeManager;
        mTrackId = trackID;
    }

    public long getNativeEffect() {
        mNativeEffect = nGetNativeEffect(mNativeManager, mTrackId, mEffectId, getType().ordinal());
        return mNativeEffect;
    }


    static native long nGetNativeEffect(long nativeManager, String trackID, String effectID, int effectType);
    static native double nStartTime(long nativeEffect);
    static native void nSetStartTime(long nativeEffect, double startTime);
    static native double nDuration(long nativeEffect);
    static native void nSetDuration(long nativeEffect, double duration);
    static native String nGetEffectId(long nativeEffect);
    static native int nEffectType(long nativeEffect);
    static native double nResourceDuration(long nativeEffect);
    static native void nSetFollowType(long nativeEffect, int type);
    static native int nFollowType(long nativeEffect);
    static native void nSetExtendType(long mNativeEffect, int type);
    static native int nExtendType(long mNativeEffect);
    static native void nSetSpeed(long nativeEffect, float speed);
    static native float nSpeed(long mNativeEffect);
    static native double nDurationOfOneCycle(long nativeEffect);
    static native void nSetDurationOfOneCycle(long nativeEffect, double duration);

    static native void nFilterSetAlpha(long nativeFilter, float alpha);
    static native float nFilterGetAlpha(long nativeFilter);

    static native long nFilterResource(long nativeEffect);
    static native boolean nSetFilterResource(long nativeEffect, long nativeResource);

    static native long nAnimationResource(long nativeEffect);
    static native boolean nSetAnimationResource(long nativeEffect, long nativeResource);

    static native long nTextAnimationResource(long nativeEffect);
    static native boolean nSetTextAnimationResource(long nativeEffect, long nativeResource);

    static native long nTrackAnimationResource(long nativeEffect);
    static native boolean nSetTrackAnimationResource(long nativeEffect, long nativeResource);

    static native long nVideoEffectResource(long nativeEffect);
    static native boolean nSetVideoEffectResource(long nativeEffect, long nativeResource);


    static native  boolean nGenericEffectReset(long nativeEffect);
    static native  boolean nGenericEffectSetAttributeInt(long nativeEffect, String name, int value);
    static native  boolean nGenericEffectSetAttributeFloat(long nativeEffect, String name, float value);
    static native  boolean nGenericEffectSetAttributeString(long nativeEffect, String name, String value);
    static native  boolean nGenericEffectSetAttributeColor(long nativeEffect, String name, String value);
    static native  boolean nGenericEffectAttribute(long nativeEffect, String name);

}
